@@ -28,7 +28,6 @@
 #include "core/mixer.h"
 #include "dbus/dbuscontrolwrapper.h"
 #include "gui/guiprofile.h"
-#include "core/volume.h"
 
 static const QString channelTypeToIconName( MixDevice::ChannelType type )
 {
@@ -217,16 +216,6 @@ QString MixDevice::configGroupName(QString prefix) const
 	 return devgrp;
 }
 
-/**
- * Returns a fully qualified id of this control, as a String in the form "controlId@mixerId"
- * @return
- */
-QString MixDevice::fullyQualifiedId() const
-{
-	QString fqId = QString("%1@%2").arg(_id, _mixer->id());
-	return fqId;
-}
-
 /**
  * Creates a deep copy of the given Volume, and adds it to this MixDevice.
  *
